aboutsummaryrefslogtreecommitdiffstats
path: root/src/plugins/http/http.h
diff options
context:
space:
mode:
Diffstat (limited to 'src/plugins/http/http.h')
-rw-r--r--src/plugins/http/http.h60
1 files changed, 47 insertions, 13 deletions
diff --git a/src/plugins/http/http.h b/src/plugins/http/http.h
index 7405d3d3bf7..3bdbc6cfdda 100644
--- a/src/plugins/http/http.h
+++ b/src/plugins/http/http.h
@@ -37,6 +37,18 @@
#define HTTP_DBG(_lvl, _fmt, _args...)
#endif
+typedef enum http_udp_tunnel_mode_
+{
+ HTTP_UDP_TUNNEL_CAPSULE, /**< app receive raw capsule */
+ HTTP_UDP_TUNNEL_DGRAM, /**< convert capsule to datagram (zc proxy) */
+} http_udp_tunnel_mode_t;
+
+typedef struct transport_endpt_cfg_http
+{
+ u32 timeout; /**< HTTP session timeout in seconds */
+ http_udp_tunnel_mode_t udp_tunnel_mode; /**< connect-udp mode */
+} transport_endpt_cfg_http_t;
+
typedef struct http_conn_id_
{
union
@@ -82,7 +94,8 @@ typedef enum http_conn_state_
_ (4, WAIT_TRANSPORT_METHOD, "wait transport method") \
_ (5, WAIT_APP_REPLY, "wait app reply") \
_ (6, APP_IO_MORE_DATA, "app io more data") \
- _ (7, TUNNEL, "tunnel")
+ _ (7, TUNNEL, "tunnel") \
+ _ (8, UDP_TUNNEL, "udp tunnel")
typedef enum http_req_state_
{
@@ -424,7 +437,11 @@ typedef struct http_req_
u32 rx_buf_offset; /* current offset during parsing */
u32 control_data_len; /* start line + headers + empty line */
- u64 to_recv; /* remaining bytes of message body to receive from transport */
+ union
+ {
+ u64 to_recv; /* remaining bytes of body to receive from transport */
+ u64 to_skip; /* remaining bytes of capsule to skip */
+ };
u8 is_tunnel;
@@ -477,6 +494,7 @@ typedef struct http_tc_
u8 *app_name;
u8 *host;
u8 is_server;
+ http_udp_tunnel_mode_t udp_tunnel_mode;
http_req_t req;
} http_conn_t;
@@ -493,6 +511,9 @@ typedef struct http_main_
http_conn_t *ho_conn_pool;
u32 app_index;
+ u8 **rx_bufs;
+ u8 **tx_bufs;
+
clib_timebase_t timebase;
u16 *sc_by_u16;
@@ -858,7 +879,7 @@ http_token_is_case (const char *actual, uword actual_len, const char *expected,
return 0;
for (i = 0; i < expected_len; i++)
{
- if (tolower (actual[i]) != expected[i])
+ if (tolower (actual[i]) != tolower (expected[i]))
return 0;
}
return 1;
@@ -994,8 +1015,6 @@ http_build_header_table (http_header_table_t *ht, http_msg_t msg)
header->name.len = name.len;
header->value.base = (char *) (ht->buf + field_line->value_offset);
header->value.len = field_line->value_len;
- HTTP_DBG (1, "value: %U", format_http_bytes, header->value.base,
- header->value.len);
hash_set_mem (ht->value_by_name, &header->name, header - ht->headers);
}
}
@@ -1006,7 +1025,7 @@ http_build_header_table (http_header_table_t *ht, http_msg_t msg)
* @param header_table Header table to search.
* @param name Header name to match.
*
- * @return Header's value in case of success, @c 0 otherwise.
+ * @return Header in case of success, @c 0 otherwise.
*/
always_inline const http_header_t *
http_get_header (http_header_table_t *header_table, const char *name,
@@ -1094,14 +1113,27 @@ typedef struct
u8 is_ip4;
} http_uri_t;
+/**
+ * An "authority-form" URL parsing.
+ *
+ * @param target Target URL to parse.
+ * @param target_len Length of URL.
+ * @param authority Parsed URL metadata in case of success.
+ *
+ * @return @c 0 on success.
+ */
always_inline int
-http_parse_authority_form_target (u8 *target, http_uri_t *authority)
+http_parse_authority_form_target (u8 *target, u32 target_len,
+ http_uri_t *authority)
{
unformat_input_t input;
+ u8 *tmp = 0;
u32 port;
int rv = 0;
- unformat_init_vector (&input, vec_dup (target));
+ vec_validate (tmp, target_len - 1);
+ vec_copy (tmp, target);
+ unformat_init_vector (&input, tmp);
if (unformat (&input, "[%U]:%d", unformat_ip6_address, &authority->ip.ip6,
&port))
{
@@ -1184,13 +1216,14 @@ _parse_port (u8 **pos, u8 *end, u16 *port)
/**
* An "absolute-form" URL parsing.
*
* @param url Vector of target URL to validate.
* @param parsed Parsed URL metadata in case of success.
*
* @return @c 0 on success.
*/
always_inline int
-http_parse_absolute_form (u8 *url, http_url_t *parsed)
+http_parse_absolute_form (u8 *url, u32 url_len, http_url_t *parsed)
{
u8 *token_start, *token_end, *end;
int is_encoded = 0;
@@ -1204,7 +1237,7 @@ http_parse_absolute_form (u8 *url, http_url_t *parsed)
0x0000000000000000,
};
- if (vec_len (url) < 9)
+ if (url_len < 9)
{
clib_warning ("uri too short");
return -1;
@@ -1212,7 +1245,7 @@ http_parse_absolute_form (u8 *url, http_url_t *parsed)
clib_memset (parsed, 0, sizeof (*parsed));
- end = url + vec_len (url);
+ end = url + url_len;
/* parse scheme */
if (!memcmp (url, "http:// ", 7))
@@ -1367,6 +1400,7 @@ http_parse_masque_host_port (u8 *path, u32 path_len, http_uri_t *parsed)
}
#define HTTP_INVALID_VARINT ((u64) ~0)
+#define HTTP_CAPSULE_HEADER_MAX_SIZE 8
#define HTTP_UDP_PROXY_DATAGRAM_CAPSULE_OVERHEAD 5
#define HTTP_UDP_PAYLOAD_MAX_LEN 65527
@@ -1184,13 +1216,14 @@ _parse_port (u8 **pos, u8 *end, u16 *port)
* An "absolute-form" URL parsing.
*
- * @param url Vector of target URL to validate.
- * @param parsed Parsed URL metadata in case of success.
+ * @param url Target URL to parse.
+ * @param url_len Length of URL.
+ * @param parsed Parsed URL metadata in case of success.
*
* @return @c 0 on success.